"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Debugger = require("debug");
var qs = require("qs");
var isBuffer = require("is-buffer");
var methods_1 = require("./methods");
var model_1 = require("./model");
exports.Model = model_1.default;
var result_1 = require("./result");
var debug = Debugger('akita:client');
var INSTANCES = {};
function isStream(value) {
    return (value
        && typeof value === 'object'
        && typeof value.pipe === 'function'
        && value.readable !== false
        && typeof value._readableState === 'object');
}
function isFile(value) {
    return (value
        && typeof value === 'object'
        && typeof value.slice === 'function'
        && value.size
        && value.lastModified);
}
function resolve(key) {
    if (!INSTANCES[key]) {
        INSTANCES[key] = create();
    }
    return INSTANCES[key];
}
function create(options) {
    var client = function client(path) {
        var _a;
        return _a = (function (_super) {
                __extends(AnonymousModel, _super);
                function AnonymousModel() {
                    return _super !== null && _super.apply(this, arguments) || this;
                }
                return AnonymousModel;
            }(model_1.default)),
            _a.path = path,
            _a.client = client,
            _a;
    };
    client._options = options || {};
    client._count = 0;
    client.create = create;
    client.resolve = resolve;
    client.setOptions = function (opts) {
        client._options = Object.assign({}, client._options, opts);
    };
    function getFormDataClass() {
        var FormData = client._options.FormData;
        if (!FormData && typeof window === 'object' && window.FormData) {
            FormData = window.FormData;
        }
        else if (!FormData && typeof global === 'object' && global.FormData) {
            FormData = global.FormData;
        }
        return FormData;
    }
    client.createBody = function (body) {
        var FormData = getFormDataClass();
        if (!body || typeof body !== 'object' || !FormData || isBuffer(body) || body instanceof ArrayBuffer || body instanceof FormData)
            return body;
        var form = null;
        function addField(path, value) {
            if (typeof value === 'undefined')
                return;
            if (!isStream(value) && !isFile(value)) {
                if (typeof value === 'boolean' || typeof value === 'number' || value === null || value instanceof Date) {
                    value = String(value);
                }
                else if (typeof value === 'object') {
                    for (var key in value) {
                        addField(path + '[' + key + ']', value[key]);
                    }
                    return;
                }
            }
            form.append(path, value);
        }
        for (var key in body) {
            var value = body[key];
            if (isStream(value) || isFile(value)) {
                form = new FormData();
                Object.keys(body).forEach(function (path) { return addField(path, body[path]); });
                break;
            }
        }
        return form || body;
    };
    client.request = function request(path, init, query, reducer) {
        init = Object.assign({}, init);
        var queryParams = Object.assign({}, init.query);
        delete init.query;
        var queryString = qs.stringify(queryParams);
        if (queryString) {
            path += "?" + queryString;
        }
        if (init.headers) {
            init.headers = Object.assign({}, init.headers);
        }
        if (init.body && typeof init.body === 'object') {
            init.body = client.createBody(init.body);
            var FormData_1 = getFormDataClass();
            if (!isBuffer(init.body) && !(FormData_1 && init.body instanceof FormData_1) && !(init.body instanceof ArrayBuffer)) {
                if (!init.headers) {
                    init.headers = {};
                }
                if (!init.headers['Content-Type'] || init.headers['Content-Type'].indexOf('json') > -1) {
                    if (!init.headers['Content-Type']) {
                        init.headers['Content-Type'] = 'application/json';
                    }
                    init.body = JSON.stringify(init.body);
                }
                else if (init.headers['Content-Type'] === 'application/x-www-form-urlencoded') {
                    init.body = qs.stringify(init.body);
                }
                else {
                    throw new Error("Akita Error: Unsupported Content-Type " + init.headers['Content-Type']);
                }
            }
        }
        if (client._options.init) {
            init = Object.assign({}, client._options.init, init);
            if (client._options.init.headers) {
                init.headers = Object.assign({}, client._options.init.headers, init.headers);
            }
        }
        if (!init.headers) {
            init.headers = {};
        }
        var apiRoot = client._options.apiRoot;
        if (apiRoot) {
            if (apiRoot[apiRoot.length - 1] === '/' && path[0] === '/') {
                path = path.substring(1);
            }
            else if (apiRoot[apiRoot.length - 1] !== '/' && path[0] !== '/') {
                path = "/" + path;
            }
            path = apiRoot + path;
        }
        if (debug.enabled) {
            debug(init.method, path, JSON.stringify(init));
        }
        var fetch = client._options.fetch;
        if (!fetch) {
            if (typeof window !== 'undefined') {
                fetch = window.fetch;
            }
            else if (typeof global !== 'undefined') {
                fetch = global.fetch;
            }
        }
        client._count += 1;
        return new result_1.default(fetch, path, init, query, reducer);
    };
    methods_1.default.forEach(function (method) {
        client[method] = function (path, init) {
            init = init || {};
            init.method = method.toUpperCase();
            return client.request(path, init);
        };
    });
    return client;
}
exports.default = resolve('default');
