"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var form_data_1 = require("./form-data");
var headers_1 = require("./headers");
function fetch(url, init) {
    return new Promise(function (resolve, reject) {
        var response = {
            url: url,
            ok: true,
            redirected: false,
            status: 200,
            statusText: 'OK',
            headers: new headers_1.default(),
            _result: null,
            get body() {
                throw new Error('Akita streaming not support for Weixin');
            },
            get blob() {
                throw new Error('Akita streaming not support for Weixin');
            },
            arrayBuffer: function () {
                return Promise.resolve(response._result);
            },
            json: function () {
                return response.text().then(function (text) { return JSON.parse(text); });
            },
            text: function () {
                if (typeof response._result === 'string') {
                    return Promise.resolve(response._result);
                }
                var arr = new Uint8Array(response._result);
                var str = String.fromCharCode.apply(String, arr);
                str = decodeURIComponent(escape((str)));
                return Promise.resolve(str);
            }
        };
        var req = {
            method: init.method,
            url: url,
            header: init.headers,
            dataType: 'text',
            responseType: 'arraybuffer'
        };
        if (req.method === 'PATCH') {
            if (!req.header) {
                req.header = {};
            }
            req.method = 'PUT';
            req.header['akita-method'] = 'PATCH';
        }
        var fn = 'request';
        if (init.body instanceof form_data_1.default) {
            for (var key in init.body) {
                var value = init.body[key];
                if (value && typeof value === 'object' && value.filePath && Object.keys(value).length === 1) {
                    req.name = key;
                    req.filePath = value.filePath;
                    delete init.body[key];
                }
            }
            if (req.filePath) {
                fn = 'uploadFile';
                req.formData = init.body;
            }
        }
        if (fn === 'request') {
            req.data = init.body;
        }
        req.success = function (res) {
            response._result = res.data;
            if (res.header) {
                for (var key in res.header) {
                    response.headers.append(key, res.header[key]);
                }
            }
            response.status = res.statusCode;
            if (response.status !== 200) {
                response.statusText = 'Unknown';
            }
            if (response.status >= 400) {
                response.ok = false;
            }
            resolve(response);
        };
        req.fail = function (res) {
            if (res && res.errMsg) {
                reject(new Error(res.errMsg));
            }
            else {
                reject(res);
            }
        };
        wx[fn](req);
    });
}
exports.default = fetch;
